from __future__ import annotations

from typing import Any, Callable, TypeVar

F = TypeVar("F", bound=Callable[..., Any])


try:
    import wandb
    import weave
except ImportError:
    wandb = None
    weave = None


def op(*dargs: Any, **dkwargs: Any):
    if weave is None:
        if dargs and callable(dargs[0]) and len(dargs) == 1 and not dkwargs:
            return dargs[0]

        def _decorator(fn: F) -> F:
            return fn

        return _decorator
    return weave.op(*dargs, **dkwargs)


def init(project: str, api_key: str | None = None) -> None:
    if wandb is not None and api_key:
        wandb.login(key=api_key)
    if weave is not None:
        weave.init(project)



import pinetext.telemetry as telemetry


def test_op():
    @telemetry.op()
    def f(x: int) -> int:
        return x + 1

    assert f(1) == 2


def test_init(monkeypatch):
    calls = {"login": 0, "init": 0}

    class DummyWandB:
        def login(self, key: str):
            calls["login"] += 1
            assert key == "secret"

    class DummyWeave:
        def init(self, project: str):
            calls["init"] += 1
            assert project == "pinetext-proj"

    monkeypatch.setattr(telemetry, "wandb", DummyWandB())
    monkeypatch.setattr(telemetry, "weave", DummyWeave())

    telemetry.init("pinetext-proj", api_key="secret")
    assert calls["login"] == 1
    assert calls["init"] == 1
